@@ -65,8 +65,8 @@ Internal function.
 Turn both `a` and `::constraint` into `a::Annotation{<:constraint}` etc
 """
 function _constrain_and_name(arg::Expr)
-    Meta.isexpr(arg, :(::), 2) && return Expr(:(::), Symbol("ann_", arg.args[1]), :(Annotation{<:$(arg.args[2])})) # it is already fine.
-    Meta.isexpr(arg, :(::), 1) && return Expr(:(::), Symbol("ann_", gensym()), :(Annotation{<:$(arg.args[1])})) # add name
+    Meta.isexpr(arg, :(::), 2) && return Expr(:(::), Symbol("ann_", arg.args[1]), :($(Annotation){<:$(arg.args[2])})) # it is already fine.
+    Meta.isexpr(arg, :(::), 1) && return Expr(:(::), Symbol("ann_", gensym()), :($(Annotation){<:$(arg.args[1])})) # add name
     # Meta.isexpr(arg, :(...), 1) &&
     #     return Expr(:(...), _constrain_and_name(arg.args[1], :Annotation))
     return error("malformed arguments: $arg")
@@ -236,7 +236,11 @@ function scalar_frule_expr(__source__, f, call, setup_stmts, inputs, input_names
                             end
 
                             if !seen
-                                push!(gensetup, Expr(:(=), outexpr, nothing))
+                                ST = $(esc(:RT))
+                                if ST <: Tuple
+                                    ST = ST.parameters[o]
+                                end
+                                push!(gensetup, Expr(:(=), outexpr, ST))
                                 seen = true
                             end
 
@@ -473,21 +477,25 @@ function scalar_rrule_expr(__source__, f, call, setup_stmts, inputs, input_names
             end
             push!(gensetup, Expr(:(=), :cache, Expr(:tuple, caches...)))
 
+            PT = EnzymeRules.primal_type(config, ($(esc(:RTA))).parameters[1])
+            ST = EnzymeRules.shadow_type(config, ($(esc(:RTA))).parameters[1])
+            AugmentedReturnType = :($(EnzymeRules.AugmentedReturn){$PT, $ST, typeof(cache)})
+
             genres = if needs_primal(config)
                 if needs_shadow(config)
                     if width(config) == 1
-                        Expr(:call, EnzymeRules.AugmentedReturn, :Ω, :dΩ, :cache)
+                        Expr(:call, AugmentedReturnType, :Ω, :dΩ, :cache)
                     else
-                        Expr(:call, EnzymeRules.AugmentedReturn, :Ω, :dΩ, :cache)
+                        Expr(:call, AugmentedReturnType, :Ω, :dΩ, :cache)
                     end
                 else
-                    Expr(:call, EnzymeRules.AugmentedReturn, :Ω, nothing, :cache)
+                    Expr(:call, AugmentedReturnType, :Ω, nothing, :cache)
                 end
             else
                 if needs_shadow(config)
-                    Expr(:call, EnzymeRules.AugmentedReturn, nothing, :dΩ, :cache)
+                    Expr(:call, AugmentedReturnType, nothing, :dΩ, :cache)
                 else
-                    Expr(:call, EnzymeRules.AugmentedReturn, nothing, nothing, :cache)
+                    Expr(:call, AugmentedReturnType, nothing, nothing, :cache)
                 end
             end
 
@@ -613,7 +621,7 @@ function scalar_rrule_expr(__source__, f, call, setup_stmts, inputs, input_names
 
                             if !seen
                                 if inp_types[inum] <: Active
-                                    push!(gensetup, Expr(:(=), inexpr, nothing))
+                                    push!(gensetup, Expr(:(=), inexpr, eltype(inp_types[inum])))
                                 else
                                     dexpr = Expr(:call, getfield, Symbol(inp_names[inum]), 2)
                                     if W != 1
@@ -719,6 +727,31 @@ If a specific argument has no partial derivative, then all corresponding argumen
              ...)
 ```
 
+# Examples
+
+Let's write an `@easy_rule` for a simple trigonometric function. Enzyme already has rules for `sin` and `cos`, but for the sake of illustration we can define a new pass-through function to oen of them to demostrate the `@easy_rule` interface.
+
+```julia
+mycos(x) = cos(x)
+
+# forward-mode rule for the new function
+EnzymeRules.@easy_rule(
+    mycos(x::AbstractFloat),
+    @setup(),
+    (-sin(x),)
+)
+```
+
+Then this rule can be tested by running `Enzyme.autodiff` and comparing with the result for the regular `cos` function:
+
+```julia
+myderiv   = autodiff(Forward, mycos, Duplicated(2.0f0, 1.2f0))[1] # -1.091157f0
+truederiv = autodiff(Forward, cos, Duplicated(2.0f0, 1.2f0))[1] # -1.091157f0
+@assert myderiv = truederiv
+```
+
+For more information about easy rules, see the [manual](@ref man-easy-rule).
+
 """
 macro easy_rule(call, maybe_setup, partials...)
     call, setup_stmts, inputs, input_names, normal_inputs, partials = _normalize_scalarrules_macro_input(
@@ -735,4 +768,4 @@ macro easy_rule(call, maybe_setup, partials...)
         $(frule_expr)
         $(rrule_expr)
     end
-end
\ No newline at end of file
+end
